<?php

namespace Drupal\advanced_content_blocks\Form;

use Drupal\advanced_content_blocks\BlockManager;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for configuring Advanced Menu Widget settings.
 */
class AdvancedContentBlocksSettingsForm extends ConfigFormBase {

  /**
   * The block manager service.
   *
   * @var \Drupal\advanced_content_blocks\BlockManager
   */
  protected $blockManager;

  /**
   * Constructs an AdvancedMenuWidgetSettingsForm object.
   *
   * @param \Drupal\advanced_content_blocks\BlockManager $block_manager
   *   The block manager service.
   */
  public function __construct(BlockManager $block_manager) {
    $this->blockManager = $block_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
    $container->get('advanced_content_blocks.block_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'advanced_content_blocks.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'advanced_content_blocks_settings_form';
  }

  /**
   * Builds the configuration form for Advanced Menu Widget settings.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array
   *   The modified form array.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $form['block_types'] = [
      '#type' => 'radios',
      '#title' => $this->t('Select a block type to add'),
      '#options' => [
        'calendar_block' => $this->t('Calendar Block'),
        'audio_block' => $this->t('Audio Block'),
        'image_block' => $this->t('Image Block'),
        'social_links_block' => $this->t('Social Links Block'),
        'contact_us_block' => $this->t('Contact Us Block'),
    // 'custom_html_block' => $this->t('Custom Html Block'),
      ],
      '#default_value' => 'calendar_block',
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Block'),
    ];

    return $form;
  }

  /**
   * Submits the configuration form for Advanced Menu Widget settings.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save audio file configuration (if applicable).
    if ($form_state->hasValue('audio_file')) {
      $this->config('advanced_content_blocks.settings')
        ->set('audio_file', $form_state->getValue('audio_file'))
        ->save();
    }

    // Handle adding the selected block.
    $plugin_id = $form_state->getValue('block_types');
    $redirect_url = $this->blockManager->addBlock($plugin_id);

    // Create a Url object from the redirect URL.
    $url = Url::fromRoute('block.admin_display');

    // Redirect to the Block Layout page.
    $form_state->setRedirectUrl($url);
  }

}
