<?php

namespace Drupal\advanced_content_blocks\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an 'Audio' Block.
 *
 * @Block(
 *   id = "audio_block",
 *   admin_label = @Translation("Audio Block"),
 * )
 */
class AudioBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * Constructs an AudioBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, FileSystemInterface $file_system, FileUrlGeneratorInterface $file_url_generator) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->fileSystem = $file_system;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('file_system'),
      $container->get('file_url_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $file_urls = [];

    if (!empty($config['audio_files'])) {
      foreach ($config['audio_files'] as $fid) {
        $file = $this->entityTypeManager->getStorage('file')->load($fid);
        if ($file) {
          $file_urls[] = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());
        }
      }
    }

    return [
      '#theme' => 'audio_block',
      '#file_urls' => $file_urls,
      '#attached' => [
        'library' => [
          'advanced_content_blocks/audio',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();
    $default_value = !empty($config['audio_files']) ? $config['audio_files'] : [];

    $form['audio_files'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Upload Audio Files'),
      '#upload_location' => 'public://audio/',
      '#default_value' => $default_value,
      '#multiple' => TRUE,
      '#upload_validators' => [
        'file_validate_extensions' => ['mp3', 'wav', 'ogg'],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $files = $form_state->getValue('audio_files');
    $fids = [];

    foreach ($files as $fid) {
      $fid = $this->uploadFile($fid);
      if ($fid) {
        $fids[] = $fid;
      }
    }

    $this->setConfigurationValue('audio_files', $fids);
  }

  /**
   * Uploads a file and returns its ID.
   *
   * @param int $fid
   *   The file ID.
   *
   * @return int|null
   *   The file ID if successful, NULL otherwise.
   */
  private function uploadFile($fid) {
    $file = $this->entityTypeManager->getStorage('file')->load($fid);
    if ($file) {
      $file->setPermanent();
      $file->save();
      return $file->id();
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'audio_files' => [],
    ];
  }

}
