<?php

namespace Drupal\advanced_content_blocks\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an 'Image' Block with multiple image upload capability.
 *
 * @Block(
 *   id = "image_block",
 *   admin_label = @Translation("Image Block"),
 * )
 */
class ImageBlock extends BlockBase implements BlockPluginInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];

    // Retrieve image file IDs from configuration.
    $image_fids = $this->configuration['image'] ?? [];

    if (!empty($image_fids)) {
      $images = [];
      foreach ($image_fids as $fid) {
        $file = File::load($fid);
        if ($file) {
          $uri = $file->getFileUri();
          $file_url = \Drupal::service('file_url_generator')->generateAbsoluteString($uri);
          $images[] = [
            '#theme' => 'image',
            '#uri' => $file_url,
            '#alt' => $file->getFilename(),
          ];
        }
      }

      // Use the 'item_list' theme to render all images in a list.
      $build['images'] = [
        '#theme' => 'item_list',
        '#items' => $images,
        '#title' => $this->t('Images'),
        '#attributes' => ['class' => ['gallery']],
      ];
    }

    // Attach the CSS file.
    $build['#attached']['library'][] = 'advanced_content_blocks/gallery';

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['image'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Images'),
      '#upload_location' => 'public://',
      '#default_value' => $this->configuration['image'] ?? [],
      '#required' => TRUE,
      '#multiple' => TRUE,
      '#upload_validators' => [
        'file_validate_extensions' => ['png jpg jpeg gif jfif webp'],
        'file_validate_size' => [50 * 1024 * 1024],
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $image_fids = $form_state->getValue('image');
    if (!empty($image_fids)) {
      // Store all file IDs.
      $this->configuration['image'] = $image_fids;
    }
    else {
      // Clear images if none selected.
      $this->configuration['image'] = [];
    }
  }

}
